#version 450

//attribute
layout (location = 0) in vec4 a_position;
layout (location = 1) in vec4 a_normal;

//varying
layout (location = 1) out vec2 v_texcoord;
layout (location = 7) out vec4 v_diffuse;

//uniform
layout (set = 0, binding = 0) uniform Block
{
    vec4 u_viewport;
    vec4 u_pixel_rotation;
    vec4 u_diffuse_color;
    vec4 u_pixel_translation_scale;
    vec4 u_pixel_anchor;
} block;

vec3 rotate_vector( vec4 quat, vec3 vec )
{
	return vec + 2.0 * cross( cross( vec, quat.xyz ) + quat.w * vec, quat.xyz );
}

void main()
{
    v_texcoord = a_normal.zw;

    v_diffuse = vec4( block.u_diffuse_color.xyz, a_position.w * block.u_diffuse_color.w );

    vec4 currentPosition = vec4( ( a_position.x / block.u_viewport.z ) * 2.0 - 1.0,
                                 ( ( block.u_viewport.w - a_position.y ) / block.u_viewport.w ) * 2.0 - 1.0,
                                 -1.0,
                                 1.0 );

    vec3 translation = rotate_vector( block.u_pixel_rotation, vec3( a_normal.xy, 0.0) );
    translation.xy *= block.u_pixel_translation_scale.zw;

    translation += rotate_vector( block.u_pixel_rotation, vec3( block.u_pixel_translation_scale.xy, 0.0 ) );

    translation.xy += block.u_post_pixel_translation.xy;

	gl_Position.x = currentPosition.x + ( translation.x / block.u_viewport.z ) * 2.0;
	gl_Position.y = currentPosition.y - ( translation.y / block.u_viewport.w ) * 2.0 ;
    gl_Position.zw = currentPosition.zw;

}
